/*  Botik::Key Pro - Botik Key Client for Windows 9x/ME/NT/2000/XP/2003
 *  Copyright (C) 2004 Alexander V. Melnikoff
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public Licensse as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __CMNFUNCS_H__
#define __CMNFUNCS_H__

#include <stdafx.h>		// To use add $(SolutionDir)/<project/dir> as additional include path.
#include <windows.h>
#include <tchar.h>
#include <stdio.h>
#include <stdarg.h>
#include <wchar.h>
#include <commctrl.h>
#include <commdlg.h>

////////////////////////////////////////////////////////////////////////
// Debug functions and definitions.
////////////////////////////////////////////////////////////////////////

#ifndef cfDONT_USE_DPRINTF

#ifndef cfDPRINTF_APP_NAME
  #pragma message ("  **** You can customize debug output string preffix by defining cfDPRINTF_APP_NAME before cmnfuncs.h inclusion.")
  #define cfDPRINTF_APP_NAME		_T("cfDPRINTF_APP_NAME")
#endif

#define cfDPRINTF_OUTPUT_FORMAT _T("%s: %s")

VOID cfdprintf(LPCTSTR szFormat_, ...);
#else  // cfDONT_USE_DPRINTF
inline VOID cfdprintf(LPCTSTR, ...) {/* empty */};
#endif // cfDONT_USE_DPRINTF

#ifdef _DEBUG
#define cfBREAK_PT __asm int 3;
#else
#define cfBREAK_PT
#endif

////////////////////////////////////////////////////////////////////////
// Miscelaneous...
////////////////////////////////////////////////////////////////////////

#define cfCHECK_RET(a) if (a != ERROR_SUCCESS) return(FALSE);

#define cfTCBLENGTH(buf) (sizeof(buf)/sizeof(TCHAR))

#define cfSafeRelease(p)	\
{							\
    if(p)					\
        (p)->Release();		\
    (p) = NULL;				\
};

////////////////////////////////////////////////////////////////////////
// File operations...
////////////////////////////////////////////////////////////////////////
DWORD cfReadLine(HANDLE hFile_, LPTSTR* lpszBuf_, DWORD dwBufSize_, LPDWORD pdwOutBufSize_);

////////////////////////////////////////////////////////////////////////
// Memory operations...
////////////////////////////////////////////////////////////////////////

inline void* cfRealloc(void* ptr_, size_t buflen_, size_t newlen_)
{
	unsigned char* buf;

	cfdprintf(_T("cfrealloc: old size: %d; new size: %d"), buflen_, newlen_);

	buf = new unsigned char[newlen_];

	buf[0] = 0;

	if(ptr_ != NULL && buflen_ > 0)
	{
		memcpy(buf, ptr_, buflen_);
		delete[] ptr_;
	};

	cfdprintf(_T("!cfrealloc"));

	return(buf);
};

////////////////////////////////////////////////////////////////////////
// User notification and logging functions.
////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////
// To use message call (LPTSTR)LocalLock(hLocal)
// Then LocalFree(hLocal) to free message buffer.
// LANG_ENGLISH, SUBLANG_ENGLISH_US
inline VOID cfGetFormatMessage(HLOCAL& hLocal_, DWORD dwErrId_, WORD wLang_ = LANG_NEUTRAL, WORD wSubLang_ = SUBLANG_DEFAULT) 
{
    FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM|FORMAT_MESSAGE_ALLOCATE_BUFFER, 
				  NULL, dwErrId_, MAKELANGID(wLang_,wSubLang_), 
				  (LPTSTR)&hLocal_, 0, NULL);
};

#define cfEVENT_ALERT_TEXT_FORNAT		_T("%s\n\nCode: %d (0x%08x)\nError: %s")

VOID cfAlertBox(UINT uType_, LPCTSTR szCapt_, LPCTSTR szFormat_, ...);

VOID cfAlertBoxError(LPCTSTR szCapt_, LPCTSTR szText_, DWORD dwErr_);

VOID cfAlertBoxWarning(LPCTSTR szCapt_, LPCTSTR szText_, DWORD dwErr_);

VOID cfLogEvent(LPCTSTR szSvcName_, WORD wType_, DWORD dwEvtID_, LPCTSTR szFormat_ = NULL, ...);

VOID cfLogEventError(LPCTSTR szSvcName_, DWORD dwEvtID_, LPCTSTR szText_, DWORD dwErr_);

VOID cfLogEventWarning(LPCTSTR szSvcName_, DWORD dwEvtID_, LPCTSTR szText_, DWORD dwErr_);

VOID cfInformError(BOOL fLogEvt_, LPCTSTR szCapt_, LPCTSTR szSvcName_, DWORD dwEvtID_, LPCTSTR szText_, DWORD dwErr_);

VOID cfInformWarning(BOOL fLogEvt_, LPCTSTR szCapt_, LPCTSTR szSvcName_, DWORD dwEvtID_, LPCTSTR szText_, DWORD dwErr_);

////////////////////////////////////////////////////////////////////////
// Common dialogs...
////////////////////////////////////////////////////////////////////////
BOOL ShowOpenFileDlg(HWND hWnd_, LPTSTR szFile_, LONG nFileSize_, LPTSTR szFilter_, LPTSTR szTitle_, DWORD dwFlags_, LPTSTR szDefExt_ = NULL);
BOOL ShowSaveFileDlg(HWND hWnd_, LPTSTR szFile_, LONG nFileSize_, LPTSTR szFilter_, LPTSTR szTitle_, DWORD dwFlags_, LPTSTR szDefExt_ = NULL);

////////////////////////////////////////////////////////////////////////
// Windowing functions.
////////////////////////////////////////////////////////////////////////

LPTSTR cfListView_GetItemText(HWND hwnd_, INT nItm_, INT nSubItm_);

LONG cfGetDlgItemLong(HWND hDlg_, UINT uID_, INT nIndex_);

LONG cfSetDlgItemLong(HWND hDlg_, UINT uID_, INT nIndex_, LONG lNewLong_);

BOOL cfEnableDlgItem(HWND hDlg_, UINT uID_, BOOL fEnable_);

INT cfGetDlgItemTextLength(HWND hDlg_, UINT uID_);

////////////////////////////////////////////////////////////////////////
// Registry functions.
////////////////////////////////////////////////////////////////////////

INT cfRegReadDWVal(HKEY hKey_, LPTSTR szName_, LPDWORD pdwVal_, DWORD dwDef_);

INT cfRegReadSTRVal(HKEY hKey_, LPTSTR szName_, LPTSTR* szVal_, LPTSTR szDef_);

INT cfRegSaveDWVal(HKEY hKey_, LPTSTR szName_, DWORD dwVal_);

INT cfRegSaveSTRVal(HKEY hKey_, LPTSTR szName_, LPTSTR szVal_);



#endif // __CMNFUNCS_H__